@@ -6,36 +6,36 @@ can be applied to environments to modify them in useful ways (e.g., shuffling, s
 """
 
 from coba.environments.core       import Environments
-from coba.environments.primitives import Context, Action, Interaction, Environment, SafeEnvironment
-from coba.environments.filters    import Shuffle, Take, Identity, Reservoir, Riffle
-from coba.environments.filters    import Sort, Scale, Cycle, Impute
-from coba.environments.filters    import Binary, Warm, Sparse, Where, Noise
-from coba.environments.filters    import EnvironmentFilter
+from coba.environments.primitives import Context, Action, Actions, Reward, Feedback
+from coba.environments.primitives import Interaction, Environment, SafeEnvironment
+from coba.environments.primitives import SimulatedInteraction, LoggedInteraction, GroundedInteraction
+from coba.environments.primitives import Reward, L1Reward, HammingReward, ScaleReward, BinaryReward
+from coba.environments.primitives import SequenceReward, MulticlassReward
+from coba.environments.primitives import Feedback, SequenceFeedback, EnvironmentFilter
+from coba.environments.filters    import Shuffle, Take, Identity, Reservoir, Riffle, Cache
+from coba.environments.filters    import Sort, Scale, Cycle, Impute, Flatten, Params
+from coba.environments.filters    import Binary, Warm, Sparse, Where, Noise, Grounded
+from coba.environments.filters    import Repr, Finalize, Batch, BatchSafe, Chunk
 
-from coba.environments.simulated.primitives import SimulatedInteraction, SimulatedEnvironment
-from coba.environments.simulated.synthetics import MemorySimulation, LambdaSimulation
-from coba.environments.simulated.synthetics import LinearSyntheticSimulation, NeighborsSyntheticSimulation
+from coba.environments.simulated.synthetics import LambdaSimulation, LinearSyntheticSimulation, NeighborsSyntheticSimulation
 from coba.environments.simulated.synthetics import KernelSyntheticSimulation, MLPSyntheticSimulation
 from coba.environments.simulated.openml     import OpenmlSimulation, OpenmlSource
 from coba.environments.simulated.supervised import SupervisedSimulation, CsvSource, ArffSource, LibSvmSource, ManikSource
 from coba.environments.simulated.serialized import SerializedSimulation
 
-from coba.environments.logged.primitives import LoggedInteraction, LoggedEnvironment
-from coba.environments.warmstart.primitives import WarmStartEnvironment
-
 __all__ = [
     'Context',
     'Action',
+    'Actions',
+    'Reward',
+    'Feedback',
     'Interaction',
     'SimulatedInteraction',
     'LoggedInteraction',
+    'GroundedInteraction',
     'Environment',
     'SafeEnvironment',
-    'SimulatedEnvironment',
-    'LoggedEnvironment',
-    'WarmStartEnvironment',
     'Environments',
-    'MemorySimulation',
     'LambdaSimulation',
     'OpenmlSimulation',
     'OpenmlSource',
@@ -63,5 +63,22 @@ __all__ = [
     'Noise',
     'Identity',
     'Sparse',
-    'Riffle'
+    'Riffle',
+    'Flatten',
+    'Params',
+    'Cache',
+    'Grounded',
+    'L1Reward', 
+    'HammingReward', 
+    'ScaleReward',
+    'BinaryReward',
+    'SequenceReward',
+    'Feedback',
+    'SequenceFeedback',
+    'Finalize',
+    'Repr',
+    'MulticlassReward',
+    'Batch',
+    'BatchSafe',
+    'Chunk'
 ]
