@@ -4,40 +4,34 @@ import json
 from typing import Tuple, Sequence, Any, Iterable, Dict, MutableSequence, MutableMapping, Union, overload
 
 from coba.random import random
-from coba.pipes import Pipes, Source, HttpSource, Drop, ArffReader, ListSource, Structure
-from coba.pipes.readers import SparseWithMeta
-from coba.contexts import CobaContext, CobaContext
+from coba.pipes import Pipes, Source
+from coba.pipes import HttpSource, ArffReader, DropRows, LabelRows
+from coba.contexts import CobaContext
+from coba.primitives import Sparse, Dense
 from coba.exceptions import CobaException
 
 from coba.environments.simulated.supervised import SupervisedSimulation
 
 class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],Any]]]):
     """Load a source from openml.org (or from disk if previously cached).
-
     This is primarily used by OpenmlSimulation to create Environments for Experiments.
     """
 
     @overload
-    def __init__(self, *, data_id:int, cat_as_str:bool=False, drop_missing:bool=False, skip_structure:bool=False):
+    def __init__(self, *, data_id:int, drop_missing:bool=True):
         """Instantiate an OpenmlSource.
-
         Args:
             data_id: The data id uniquely identifying the dataset on openml (i.e., openml.org/d/{id})
-            cat_as_str: Categorical features should be encoded as a string rather than one hot encoded.
             drop_missing: Drop data rows with missing values.
-            skip_structure: Skip structuring the rows (i.e. return flat rows instead of tuples of label and features).
         """
         ...
 
     @overload
-    def __init__(self, *, task_id:int, cat_as_str:bool=False, drop_missing:bool=False, skip_structure:bool=False):
+    def __init__(self, *, task_id:int, drop_missing:bool=True):
         """Instantiate an OpenmlSource.
-
         Args:
             task_id: The openml task id which identifies the dataset to use from openml along with its label
-            cat_as_str: Categorical features should be encoded as a string rather than one hot encoded.
             drop_missing: Drop data rows with missing values.
-            skip_structure: Skip structuring the rows (i.e. return flat rows instead of tuples of label and features).
         """
         ...
 
@@ -47,16 +41,14 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
         self._data_id        = kwargs.get('data_id',None)
         self._task_id        = kwargs.get('task_id',None)
         self._target         = None
-        self._cat_as_str     = kwargs.get('cat_as_str',False)
         self._drop_missing   = kwargs.get('drop_missing',True)
-        self._skip_structure = kwargs.get('skip_structure',False)
 
     @property
     def params(self) -> Dict[str,Any]:
         """Parameters describing the openml source."""
-        return  { "openml_data": self._data_id, "openml_task": self._task_id, "openml_target": self._target, "cat_as_str": self._cat_as_str, "drop_missing": self._drop_missing }
+        return  { "openml_data": self._data_id, "openml_task": self._task_id, "openml_target": self._target, "drop_missing": self._drop_missing }
 
-    def read(self) -> Iterable[Tuple[Any, Any]]:
+    def read(self) -> Iterable[Union[Dense,Sparse]]:
         """Read and parse the openml source."""
 
         try:
@@ -65,7 +57,7 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
             # openml_semaphore will be None if we aren't multiprocessing otherwise it'll have a sema
             openml_semaphore = CobaContext.store.get("openml_semaphore")
             semaphore_acquired = False
-            
+
             if openml_semaphore and not self._source_already_cached():
                 openml_semaphore.acquire()
                 if self._source_already_cached(): #pragma: no cover
@@ -98,7 +90,7 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
             if data_descr.get('status') == 'deactivated':
                 raise CobaException(f"Openml {self._data_id} has been deactivated. This is often due to flags on the data.")
 
-            is_ignore = lambda feat_descr: (
+            is_ignore = lambda feat_descr:(
                 feat_descr['is_ignore'        ] == 'true' or
                 feat_descr['is_row_identifier'] == 'true' or
                 feat_descr['data_type'        ] not in ['numeric', 'nominal']
@@ -108,27 +100,15 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
 
             if self._target in ignore: ignore.pop(ignore.index(self._target))
 
-            def drop_row(row):
-                row_values = row._values.values() if isinstance(row,SparseWithMeta) else row._values
-                has_missing = "?" in row_values or "" in row_values
-                return self._drop_missing and has_missing
+            label_type = 'c' if task_type==1 else 'r' if task_type==2 else None
 
-            source    = ListSource(self._get_arff_lines(data_descr["file_id"], None))
-            reader    = ArffReader(cat_as_str=self._cat_as_str)
-            drop      = Drop(drop_cols=ignore, drop_row=drop_row)
+            drop_row  = (lambda r: r.missing) if self._drop_missing else None
+            lines     = self._get_arff_lines(data_descr["file_id"], None)
+            reader    = ArffReader()
+            drop      = DropRows(drop_cols=ignore, drop_row=drop_row)
+            label     = LabelRows(self._target, label_type)
 
-            if self._skip_structure:
-                for raw_output in Pipes.join(source, reader, drop).read():
-                    yield raw_output
-            else:
-                structure = Structure([None, self._target])
-
-                for features, label in Pipes.join(source, reader, drop, structure).read():
-                    # ensure that SupervisedSimulation will interpret the label as a class
-                    if task_type == 1 and isinstance(label,(int,float)): 
-                        label = str(int(label) if float(label).is_integer() else label)
-                    
-                    yield features, label
+            return Pipes.join(reader, drop, label).filter(lines)
 
         except KeyboardInterrupt:
             #we don't want to clear the cache in the case of a KeyboardInterrupt
@@ -142,7 +122,7 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
             #if something unexpected went wrong clear the cache just in case it was corrupted somehow
             self._clear_cache()
             raise
-    
+
         finally:
             if semaphore_acquired:
                 openml_semaphore.release()
@@ -151,25 +131,14 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
         return name.strip().strip('\'"').replace('\\','') if name else name
 
     def _get_data(self, url:str, key:str, checksum:str=None) -> Iterable[str]:
-
-        # This can't be done in a streaming manner unless there is a persistent cacher.
-        # Because we don't require cacher this means this may not be possible with streaming.
-        # if checksum is not None and md5(bites).hexdigest() != checksum:
-        #     #if the cache has become corrupted we need to clear it
-        #     CobaContext.cacher.rmv(key)
-        #     message = (
-        #         f"The response from {url} did not match the given checksum {checksum}. This could be the result "
-        #         "of network errors or the file becoming corrupted. Please consider downloading the file again. "
-        #         "If the error persists you may want to manually download and reference the file.")
-        #     raise CobaException(message) from None
         try:
             for b in CobaContext.cacher.get_put(key, lambda: self._http_request(url)):
-                yield b.decode('utf-8')
+                yield b
         except Exception:
             self._clear_cache()
             raise
 
-    def _http_request(self, url:str) -> Iterable[bytes]:
+    def _http_request(self, url:str) -> Iterable[str]:
         api_key = CobaContext.api_keys['openml']
         semaphore = CobaContext.store.get("openml_semaphore")
 
@@ -215,7 +184,7 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
             # if '' == response.text:
             #     raise CobaException("Openml experienced an unexpected error. Please try requesting the data again.") from None
 
-            for b in response.iter_lines(decode_unicode=False):
+            for b in response.iter_lines(decode_unicode=True):
                 yield b
 
     def _get_data_descr(self, data_id:int) -> Dict[str,Any]:
@@ -284,29 +253,24 @@ class OpenmlSource(Source[Iterable[Tuple[Union[MutableSequence, MutableMapping],
 
 class OpenmlSimulation(SupervisedSimulation):
     """A supervised simulation created from an openml dataset.
-
     Download a dataset from openml.org and create a SupervisedSimulation.
     """
 
     @overload
-    def __init__(self, data_id: int, cat_as_str: bool = False, keep_missing: bool = False, take: int = None):
+    def __init__(self, data_id: int, drop_missing: bool = True, take: int = None):
         """Instantiate an OpenmlSimulation.
-
         Args:
             data_id: The data id uniquely identifying the dataset on openml (i.e., openml.org/d/{id})
-            cat_as_str: Indicates if categorical features should be encoded as a string rather than one hot encoded.
             drop_missing: Drop data rows with missing values.
             take: The number of interactions we'd like the simulation to have (these will be selected at random).
         """
         ...
 
     @overload
-    def __init__(self, *, task_id: int, cat_as_str: bool = False, keep_missing: bool = False, take: int=None):
+    def __init__(self, *, task_id: int, drop_missing: bool = True, take: int=None):
         """Instantiate an OpenmlSimulation.
-
         Args:
             task_id: The openml task id which identifies the dataset to use from openml along with its label
-            cat_as_str: Indicates if categorical features should be encoded as a string rather than one hot encoded.
             drop_missing: Drop data rows with missing values.
             take: The number of interactions we'd like the simulation to have (these will be selected at random).
         """
@@ -314,7 +278,7 @@ class OpenmlSimulation(SupervisedSimulation):
 
     def __init__(self, *args, **kwargs) -> None:
         """Instantiate an OpenmlSimulation."""
-        kwargs.update(zip(['data_id','cat_as_str','drop_missing','take'], args))
+        kwargs.update(zip(['data_id','drop_missing','take'], args))
         super().__init__(OpenmlSource(**kwargs), None, None, kwargs.get('take',None))
 
     @property
@@ -326,8 +290,6 @@ class OpenmlSimulation(SupervisedSimulation):
         params = [
             f"data={self.params['openml_data']}" if self.params.get('openml_data') else f"task={self.params['openml_task']}",
             f"target={self.params['openml_target']}" if self.params.get('openml_target') else "",
-            f"cat_as_str={self.params['cat_as_str']}",
-            f"drop_missing={self.params['drop_missing']}"
         ]
 
         return f"Openml({str.join(', ', filter(None,params))})"
