@@ -1,16 +1,15 @@
-import collections.abc
-
-from itertools import chain, repeat
+from itertools import chain, count
 from typing import Any, Iterable, Union, Sequence, overload, Dict, MutableSequence, MutableMapping
 from coba.backports import Literal
 
-from coba.encodings import OneHotEncoder
-from coba.random import CobaRandom
-from coba.pipes import Pipes, Source, ListSource, Structure, Reservoir, UrlSource, CsvReader
+from coba.pipes import Pipes, Source, IterableSource, LabelRows, Reservoir, UrlSource, CsvReader
 from coba.pipes import CsvReader, ArffReader, LibsvmReader, ManikReader
-from coba.statistics import percentile
 
-from coba.environments.simulated.primitives import SimulatedEnvironment, SimulatedInteraction
+from coba.utilities import peek_first
+from coba.primitives import Categorical
+
+from coba.environments.primitives import SimulatedEnvironment, SimulatedInteraction
+from coba.environments.primitives import L1Reward, MulticlassReward, HammingReward
 
 class CsvSource(Source[Iterable[MutableSequence]]):
     """Load a source (either local or remote) in CSV format.
@@ -49,21 +48,15 @@ class ArffSource(Source[Union[Iterable[MutableSequence], Iterable[MutableMapping
 
     def __init__(self,
         source: Union[str,Source[Iterable[str]]],
-        cat_as_str: bool = False,
-        skip_encoding: bool = False,
-        lazy_encoding: bool = True,
-        header_indexing: bool = True) -> None:
+        cat_as_str: bool = False) -> None:
         """Instantiate an ArffSource.
 
         Args:
             source: The data source. Accepts either a string representing the source location or another Source.
             cat_as_str: Indicates that categorical features should be encoded as a string rather than one hot encoded.
-            skip_encoding: Indicates that features should not be encoded (this means all features will be strings).
-            lazy_encoding: Indicates that features should be encoded lazily (this can save time if rows will be dropped).
-            header_indexing: Indicates that header data should be preserved so rows can be indexed by header name.
         """
         source = UrlSource(source) if isinstance(source,str) else source
-        reader = ArffReader(cat_as_str, skip_encoding, lazy_encoding, header_indexing)
+        reader = ArffReader(cat_as_str)
         self._source = Pipes.join(source, reader)
 
     def read(self) -> Union[Iterable[MutableSequence], Iterable[MutableMapping]]:
@@ -141,7 +134,7 @@ class SupervisedSimulation(SimulatedEnvironment):
     def __init__(self,
         source: Source = None,
         label_col: Union[int,str] = None,
-        label_type: Literal["C","R"] = None,
+        label_type: Literal["c","r","m"] = None,
         take: int = None) -> None:
         """Instantiate a SupervisedSimulation.
 
@@ -160,7 +153,7 @@ class SupervisedSimulation(SimulatedEnvironment):
     def __init__(self,
         X: Sequence[Any],
         Y: Sequence[Any],
-        label_type: Literal["C","R"] = None) -> None:
+        label_type: Literal["c","r","m"]) -> None:
         """Instantiate a SupervisedSimulation.
 
         Args:
@@ -179,15 +172,15 @@ class SupervisedSimulation(SimulatedEnvironment):
             label_type = args[2] if len(args) > 2 else kwargs.get("label_type", None)
             take       = args[3] if len(args) > 3 else kwargs.get("take", None)
             if take      is not None: source = Pipes.join(source, Reservoir(take))
-            if label_col is not None: source = Pipes.join(source, Structure((None,label_col)))
+            if label_col is not None: source = Pipes.join(source, LabelRows(label_col,label_type))
             params = source.params
 
         else:
             X          = args[0]
             Y          = args[1]
             label_type = args[2] if len(args) > 2 else kwargs.get("label_type", None)
-            source     = ListSource(list(zip(X,Y)))
             params     = {"source": "[X,Y]"}
+            source     = IterableSource(zip(X,Y))
 
         self._label_type = label_type
         self._source     = source
@@ -199,48 +192,48 @@ class SupervisedSimulation(SimulatedEnvironment):
 
     def read(self) -> Iterable[SimulatedInteraction]:
 
-        items = list(self._source.read())
+        first, rows = peek_first(self._source.read())
+        first_row_type = 0 if hasattr(first,'label') else 1
 
-        if not items: return []
+        if not rows: return []
 
-        features,labels = zip(*items)
+        first_label      = first.label if first_row_type == 0 else first[1]
+        first_label_type = first.tipe if first_row_type == 0 else None
 
-        self._label_type = self._label_type or ("R" if isinstance(labels[0], (int,float)) else "C")
-        self._params['label_type'] = self._label_type
-
-        if self._label_type == "R":
-            max_n_actions = 10
-
-            #Scale the labels so that the rewards are all in [0,1].
-            labels       = [float(l) for l in labels]
-            min_l, max_l = min(labels), max(labels)
-            labels       = [(l-min_l)/(max_l-min_l) for l in labels]
-
-            if len(labels) <= max_n_actions:
-                actions = labels
-            else:
-                actions = percentile(labels, [i/(max_n_actions+1) for i in range(1,max_n_actions+1)])
-
-            actions = sorted(list(set([round(a,3) for a in actions])))
-            values  = dict(zip(OneHotEncoder().fit_encodes(actions), actions))
-            actions = list(values.keys())
-
-            reward = lambda action,label: round(1-abs(values[action]-float(label)),3)
+        if first_label_type is None:
+            label_type = self._label_type or ("r" if isinstance(first_label, (int,float)) else "c")
         else:
-            #how can we tell the difference between featurized labels and multilabels????
-            #for now we will assume multilables will be passed in as arrays as opposed to tuples...
-            if not isinstance(labels[0], collections.abc.Hashable):
-                actions = list(chain.from_iterable(labels))
+            label_type = self._label_type or first_label_type
+
+        label_type = label_type.lower()
+        self._params['label_type'] = label_type.upper()
+
+        #these are the cases where we need to know all labels in the dataset to determine actions
+        if label_type == "m" or (label_type == "c" and not isinstance(first_label, Categorical)):
+            rows = list(rows)
+            if first_row_type == 0: labels = [r.label for r in rows]
+            if first_row_type == 1: labels = [r[1]    for r in rows]
+
+        if label_type == "r":
+            actions = []
+            reward  = L1Reward
+        elif label_type == "m":
+            actions = sorted(set(list(chain(*labels))))
+            action_indexes = dict(zip(actions,count()))
+            reward = lambda label: HammingReward(list(map(action_indexes.__getitem__,label)))
+        else:
+            if isinstance(first_label,Categorical):
+                actions = [ Categorical(l,first_label.levels) for l in sorted(first_label.levels) ]
             else:
-                actions = list(labels)
+                actions = sorted(set(labels))
+            
+            action_indexes = dict(zip(actions,count()))
+            indexes = list(range(len(actions)))
+            reward = lambda label: MulticlassReward(indexes, action_indexes[label])
 
-            is_label      = lambda action,label: action == label
-            in_multilabel = lambda action,label: isinstance(label,collections.abc.Sequence) and not isinstance(label,str) and action in label
-            reward        = lambda action,label: int(is_label(action,label) or in_multilabel(action,label))
-
-        contexts = features
-        actions  = CobaRandom(1).shuffle(sorted(set(actions)))
-        rewards  = [ [ reward(action,label) for action in actions ] for label in labels ]
-
-        for c,a,r in zip(contexts, repeat(actions), rewards):
-            yield SimulatedInteraction(c,a,r)
+        if first_row_type == 0:
+            for row in rows:
+                yield {'type':'simulated','context':row.feats,'actions':actions,'rewards':reward(row.label)}
+        else:
+            for row in rows:
+                yield {'type':'simulated','context':row[0]   ,'actions':actions,'rewards':reward(row[1])}
