@@ -6,23 +6,24 @@ import warnings
 
 from abc import ABC, abstractmethod
 from statistics import mean
-from itertools import takewhile, combinations
-from typing import Iterable, Any, Dict, Sequence, Hashable, Union
+from itertools import combinations
+from typing import Iterable, Any, Sequence, Mapping, Hashable, Union
 
 from coba.statistics import percentile
 from coba.exceptions import CobaExit, CobaException
-from coba.random import CobaRandom
 from coba.learners import Learner, SafeLearner
 from coba.encodings import InteractionsEncoder
-from coba.utilities import PackageChecker
-from coba.contexts import InteractionContext
-from coba.environments import Environment, Interaction, SimulatedInteraction, LoggedInteraction, SafeEnvironment
+from coba.utilities import PackageChecker, peek_first
+from coba.contexts import CobaContext
+from coba.primitives import Batch
+from coba.environments import Environment, SafeEnvironment
+from coba.environments import Interaction, SimulatedInteraction, LoggedInteraction, GroundedInteraction
 
 class LearnerTask(ABC):
     """A task which describes a Learner."""
 
     @abstractmethod
-    def process(self, learner: Learner) -> Dict[Any,Any]:
+    def process(self, learner: Learner) -> Mapping[Any,Any]:
         """Process the LearnerTask.
 
         Args:
@@ -37,7 +38,7 @@ class EnvironmentTask(ABC):
     """A task which describes an Environment."""
 
     @abstractmethod
-    def process(self, environment: Environment, interactions: Iterable[Interaction]) -> Dict[Any,Any]:
+    def process(self, environment: Environment, interactions: Iterable[Interaction]) -> Mapping[Any,Any]:
         """Process the EnvironmentTask.
 
         Args:
@@ -53,7 +54,7 @@ class EvaluationTask(ABC):
     """A task which evaluates a Learner on an Environment."""
 
     @abstractmethod
-    def process(self, learner: Learner, interactions: Iterable[Interaction]) -> Iterable[Dict[Any,Any]]:
+    def process(self, learner: Learner, interactions: Iterable[Interaction]) -> Iterable[Mapping[Any,Any]]:
         """Process the EvaluationTask.
 
         Args:
@@ -65,186 +66,159 @@ class EvaluationTask(ABC):
         """
         ...
 
-class OnlineOnPolicyEvalTask(EvaluationTask):
-    """Evaluate a Learner on a SimulatedEnvironment."""
-
-    def __init__(self, 
-        metrics: Union[str,Sequence[str]] = ["reward","reward_pct","rank","rank_pct","regret","regret_pct"],
-        time:bool = True, 
-        seed:int = 1) -> None:
-        """Instantiate an OnlineOnPolicyEvalTask.
+class SimpleEvaluation(EvaluationTask):
 
+    def __init__(self,
+        reward_metrics: Union[str,Sequence[str]] = "reward",
+        time_metrics: bool = False,
+        probability: bool = False) -> None:
+        """
         Args:
-            metrics: The metrics to keep track of while evaluating learners in addition to reward.
-            time: Indicate whether time statistcs should be tracked during evaluation.
-            seed: A random seed which determines which action is taken given predictions.
+            reward_metrics: The reward metrics to that should be recorded for each example.
+                The metric options include: reward, rank, regret.
+            time_metrics: Indicates whether time metrics should be recorded.
+            probability: Indicates whether the action's probability should be recorded.
         """
 
-        if not metrics:
+        if not reward_metrics:
             raise CobaException("At least one performance metric is required.")
 
-        if isinstance(metrics,str): 
-            metrics = [metrics]
+        if isinstance(reward_metrics,str): 
+            reward_metrics = [reward_metrics]
 
-        self._metrics = metrics
-        self._time    = time
-        self._seed    = seed
+        self._metrics = reward_metrics
+        self._time    = time_metrics
+        self._prob    = probability
 
-    def process(self, learner: Learner, interactions: Iterable[SimulatedInteraction]) -> Iterable[Dict[Any,Any]]:
+    def process(self, learner: Learner, interactions: Iterable[Interaction]) -> Iterable[Mapping[Any,Any]]:
 
-        random = CobaRandom(self._seed)
+        learner = SafeLearner(learner)
 
-        if not isinstance(learner, SafeLearner): learner = SafeLearner(learner)
-        if not interactions: return
+        first, interactions = peek_first(interactions)
+
+        predict  = learner.predict
+        learn    = learner.learn
+
+        discrete = first and len(first.get('actions',[])) > 0
+        batched  = isinstance(first['type'],Batch)
+
+        learning_info = CobaContext.learning_info
+
+        if 'rank' in self._metrics and not discrete:
+            warnings.warn(f"The rank metric can only be calculated for discrete environments")
+
+        calc_rank   = 'rank'   in self._metrics and discrete
+        calc_reward = 'reward' in self._metrics
+        calc_regret = 'regret' in self._metrics
+
+        get_reward = lambda reward                    : reward
+        get_regret = lambda reward, rewards           : rewards.max()-reward
+        get_rank   = lambda reward, rewards, n_actions: sorted(map(rewards.eval,range(n_actions))).index(reward)/(n_actions-1)
 
         for interaction in interactions:
 
-            InteractionContext.learner_info.clear()
+            learning_info.clear()
 
-            context = interaction.context
-            actions = interaction.actions
+            out     = {}
+            
+            tipe    = interaction['type'][0] if batched  else interaction['type']
 
-            start_time         = time.time()
-            probabilities,info = learner.predict(context, actions)
-            predict_time       = time.time() - start_time
+            if tipe == 'simulated':
 
-            action       = random.choice(actions, probabilities)
-            action_index = actions.index(action)
-            reward       = interaction.rewards[action_index]
-            probability  = probabilities[action_index]
+                context = interaction['context']
+                actions = interaction['actions']
+                rewards = interaction['rewards']
+                extra   = {k:interaction[k] for k in interaction.keys()-SimulatedInteraction.keywords }
 
-            start_time = time.time()
-            learner.learn(context, action, reward, probability, info)
-            learn_time = time.time() - start_time
-
-            learner_info     = InteractionContext.learner_info
-            interaction_info = { k:(v[action_index] if isinstance(v,(list,tuple)) else v) for k,v in interaction.kwargs.items() }
-
-            eval_stats = {
-                'reward'      : reward,
-                'max_reward'  : max(interaction.rewards),
-                'min_reward'  : min(interaction.rewards),
-                'rank'        : 1+sorted(interaction.rewards,reverse=True).index(reward),
-                'min_rank'    : 1,
-                'max_rank'    : len(set(interaction.rewards)),
-            }
-
-            eval_metrics = {
-                'reward': eval_stats['reward'],
-                'reward_pct': (eval_stats['reward']-eval_stats['min_reward'])/(eval_stats['max_reward']-eval_stats['min_reward']),
-                'rank': eval_stats['rank'],
-                'rank_pct': (eval_stats['rank']-1)/(eval_stats['max_rank']-1),
-                'regret': eval_stats['max_reward']-eval_stats['reward'],
-                'regret_pct': (eval_stats['max_reward']-eval_stats['reward'])/(eval_stats['max_reward']-eval_stats['min_reward'])
-            }
-
-            eval_times = {} if not self._time else {"predict_time": predict_time, "learn_time": learn_time}
-
-            yield {**interaction_info, **learner_info, **eval_times, **{k:eval_metrics[k] for k in self._metrics}}
-
-class OnlineOffPolicyEvalTask(EvaluationTask):
-    """Evaluate a Learner on a LoggedEnvironment."""
-
-    def __init__(self, time:bool = True) -> None:
-        self._time = time
-
-    def process(self, learner: Learner, interactions: Iterable[LoggedInteraction]) -> Iterable[Dict[Any,Any]]:
-
-        learn_time = 0
-        predict_time = 0
-
-        if not isinstance(learner, SafeLearner): learner = SafeLearner(learner)
-        if not interactions: return
-
-        for interaction in interactions:
-
-            InteractionContext.learner_info.clear()
-
-            if not interaction.actions:
-                info             = None
-                interaction_info = {}
-
-            if interaction.actions:
-                actions          = list(interaction.actions)
                 start_time       = time.time()
-                probs,info       = learner.predict(interaction.context, actions)
+                action,prob,info = predict(context, actions)
                 predict_time     = time.time()-start_time
-                interaction_info = {}
 
-            if interaction.probability and interaction.actions:
-                ratio            = probs[actions.index(interaction.action)] / interaction.probability
-                interaction_info = {'reward': ratio*interaction.reward}
+                reward = list(map(lambda r,a: r.eval(a),rewards,action)) if batched else rewards.eval(action)
 
-            for k,v in interaction.kwargs.items():
-                interaction_info[k] = v
+                start_time = time.time()
+                learn(context, actions, action, reward, prob, **info)
+                learn_time = time.time() - start_time
 
-            reveal = interaction.reward
-            prob   = interaction.probability
+            elif tipe == 'logged':
 
-            start_time = time.time()
-            learner.learn(interaction.context, interaction.action, reveal, prob, info)
-            learn_time = time.time()-start_time
+                context = interaction['context']
+                actions = interaction.get('actions')
+                rewards = interaction.get('rewards')
+                extra   = {k:interaction[k] for k in interaction.keys()-LoggedInteraction.keywords }
 
-            learner_info  = InteractionContext.learner_info
-            time_info     = {"predict_time": predict_time, "learn_time": learn_time} if self._time else {}
+                if not actions or not rewards:
+                    predict_time = None
+                    reward       = None
+                    prob         = None
+                    info         = None
+                else:
+                    start_time   = time.time()
+                    action,prob  = predict(context, actions)[:2]
+                    reward       = list(map(rewards.eval,action)) if batched else rewards.eval(action)
+                    predict_time = time.time()-start_time
 
-            yield {**interaction_info, **learner_info, **time_info}
+                start_time = time.time()
+                learn(context, actions, interaction['action'], interaction['reward'], interaction.get('probability'))
+                learn_time = time.time()-start_time
 
-class OnlineWarmStartEvalTask(EvaluationTask):
-    """Evaluate a Learner on a WarmStartEnvironment."""
+            elif tipe == 'grounded':
 
-    def __init__(self, time:bool = True, seed:int = 1) -> None:
-        """Instantiate an OnlineOnPolicyEvalTask.
+                context   = interaction['context']
+                actions   = interaction['actions']
+                rewards   = interaction['rewards']
+                feedbacks = interaction['feedbacks']
+                extra     = { k:interaction[k] for k in interaction.keys()-GroundedInteraction.keywords }
 
-        Args:
-            seed: A random seed which determines which action is taken given predictions.
-        """
-        self._seed = seed
-        self._time = time
+                start_time       = time.time()
+                action,prob,info = predict(context, actions)
+                reward           = list(map(rewards.eval,action)) if batched else rewards.eval(action)
+                feedback         = list(map(feedbacks.eval,action)) if batched else feedbacks.eval(action)
+                predict_time     = time.time()-start_time
 
-    def process(self, learner: Learner, interactions: Iterable[Interaction]) -> Iterable[Dict[Any,Any]]:
+                start_time = time.time()
+                learn(context, actions, action, feedback, prob, **info)
+                learn_time = time.time()-start_time
 
-        if not isinstance(learner, SafeLearner): learner = SafeLearner(learner)
-        if not interactions: return
+                out['feedback'] = feedback
 
-        separable_interactions = iter(self._repeat_first_simulated_interaction(interactions))
+            else:
+                raise CobaException("An unknown interaction type was received.")
 
-        logged_interactions    = takewhile(lambda i: isinstance(i,LoggedInteraction), separable_interactions)
-        simulated_interactions = separable_interactions
+            if reward is not None:
+                if not batched:
+                    if calc_reward : out['reward'] = get_reward(reward)
+                    if calc_regret : out['regret'] = get_regret(reward, rewards)
+                    if calc_rank   : out['rank'  ] = get_rank  (reward, rewards, len(actions))
+                else:
+                    if calc_reward : out['reward'] = mean(map(get_reward,reward))
+                    if calc_regret : out['regret'] = mean(map(get_regret,reward,rewards))
+                    if calc_rank   : out['rank'  ] = mean(map(get_rank  ,reward,rewards,len(actions)))
 
-        for row in OnlineOffPolicyEvalTask(time=self._time).process(learner, logged_interactions):
-            yield row
+            if self._time: out.update(predict_time=predict_time, learn_time=learn_time)
+            if self._prob and prob is not None: out.update(probability=prob)
 
-        for row in OnlineOnPolicyEvalTask(time=self._time, seed=self._seed).process(learner, simulated_interactions):
-            yield row
+            out.update(extra)
+            out.update(learning_info)
 
-    def _repeat_first_simulated_interaction(self, interactions: Iterable[Interaction]) -> Iterable[Interaction]:
+            yield out
+            learning_info.clear()
 
-        any_simulated_found = False
-
-        for interaction in interactions:
-            yield interaction
-
-            if isinstance(interaction, SimulatedInteraction) and not any_simulated_found:
-                yield interaction
-
-            any_simulated_found = any_simulated_found or isinstance(interaction, SimulatedInteraction)
-
-class SimpleLearnerTask(LearnerTask):
+class SimpleLearnerInfo(LearnerTask):
     """Describe a Learner using its name and hyperparameters."""
 
-    def process(self, item: Learner) -> Dict[Any,Any]:
+    def process(self, item: Learner) -> Mapping[Any,Any]:
         item = SafeLearner(item)
         return {"full_name": item.full_name, **item.params}
 
-class SimpleEnvironmentTask(EnvironmentTask):
+class SimpleEnvironmentInfo(EnvironmentTask):
     """Describe an Environment using its Environment and Filter parameters."""
 
-    def process(self, environment:Environment, interactions: Iterable[Interaction]) -> Dict[Any,Any]:
-        if not isinstance(environment, SafeEnvironment): environment = SafeEnvironment(environment)
-        return { k:v for k,v in environment.params.items() if v is not None }
+    def process(self, environment:Environment, interactions: Iterable[Interaction]) -> Mapping[Any,Any]:
 
-class ClassEnvironmentTask(EnvironmentTask):
+        return { k:v for k,v in SafeEnvironment(environment).params.items() if v is not None }
+
+class ClassEnvironmentInfo(EnvironmentTask):
     """Describe an Environment made from a Classification dataset.
 
     In addition to the Environment's parameters this task calculates a number of classification
@@ -252,7 +226,7 @@ class ClassEnvironmentTask(EnvironmentTask):
     finished. To make the most of this Task sklearn should be installed.
     """
 
-    def process(self, environment: Environment, interactions: Iterable[SimulatedInteraction]) -> Dict[Any,Any]:
+    def process(self, environment: Environment, interactions: Iterable[SimulatedInteraction]) -> Mapping[Any,Any]:
 
         #sources:
         #[1]: https://arxiv.org/pdf/1808.03591.pdf (lorena2019complex)
@@ -262,11 +236,11 @@ class ClassEnvironmentTask(EnvironmentTask):
 
         #[3] found that information theoretic measures and landmarking measures are most important
 
-        contexts,actions,rewards = zip(*[ (i.context, i.actions, i.rewards) for i in interactions ])
+        contexts,actions,rewards = zip(*[ (i['context'], i['actions'], i['rewards']) for i in interactions ])
         env_stats = {}
 
-        X = [ InteractionsEncoder('x').encode(x=c) for c   in contexts             ]
-        Y = [ a[r.index(max(r))]                   for a,r in zip(actions,rewards) ]
+        X = [ InteractionsEncoder('x').encode(x=c) for c in contexts ]
+        Y = [ r.argmax()                           for r in rewards  ]
         X = self._dense(X)
 
         classes = list(set(Y))
@@ -282,7 +256,7 @@ class ClassEnvironmentTask(EnvironmentTask):
         entropy_Y = self._entropy(Y)
         entropy_X = [self._entropy(x) for x in X_bin_by_f]
 
-        mutual_XY_infos  = sorted([self._mutual_info(x,Y)              for  x in X_bin_by_f                ], reverse=True)
+        mutual_XY_infos  = sorted([self._mutual_info(x,Y) for x in X_bin_by_f], reverse=True)
 
         #Information-Theoretic Meta-features
         env_stats["class_count"          ] = k
@@ -318,7 +292,7 @@ class ClassEnvironmentTask(EnvironmentTask):
             PackageChecker.sklearn("ClassEnvironmentTask.process")
 
             import numpy as np
-            
+
             from sklearn.decomposition import TruncatedSVD
             from sklearn.neighbors import KNeighborsClassifier
             from sklearn.naive_bayes import GaussianNB
@@ -390,7 +364,7 @@ class ClassEnvironmentTask(EnvironmentTask):
         except CobaExit:
             pass
 
-        return { **SimpleEnvironmentTask().process(environment, interactions), **env_stats }
+        return { **SimpleEnvironmentInfo().process(environment, interactions), **env_stats }
 
     def _entropy(self, items: Sequence[Hashable]) -> float:
         return -sum([count/len(items)*math.log2(count/len(items)) for count in collections.Counter(items).values()])
@@ -430,7 +404,7 @@ class ClassEnvironmentTask(EnvironmentTask):
                 xf = x[f]
                 X_by_fy[f][y].append(xf)
                 X_by_f[f].append(xf)
-        
+
         mean_f  = { f:      mean(X_by_f[f])                      for f in feats}
         mean_fy = { f: { y: mean(X_by_fy[f][y]) for y in Y_set } for f in feats} 
 
@@ -451,14 +425,14 @@ class ClassEnvironmentTask(EnvironmentTask):
 
         try:
             PackageChecker.sklearn('')
-            
+
             import numpy as np
             from sklearn.covariance import shrunk_covariance
 
             Y_set = set(Y)
             X     = np.array(X).T #transpose so that equations align with paper
             Y     = np.array(Y)
-            
+
             X_by_y = { y: X[:,Y==y]  for y in Y_set }
             OVO    = []
 
