@@ -2,15 +2,16 @@ import re
 
 from sys import platform
 from itertools import repeat, compress
-from typing import Any, Dict, Union, Sequence, Optional, Tuple
+from typing import Any, Dict, Union, Sequence, Mapping, Optional, Tuple
 from coba.backports import Literal
 
 from coba.pipes import Flatten
 from coba.exceptions import CobaException
 from coba.utilities import PackageChecker
 from coba.environments import Context, Action
+from coba.primitives import Sparse, Dense
 
-from coba.learners.primitives import Learner, Probs, Info
+from coba.learners.primitives import Learner, Probs, Actions
 
 Feature       = Union[str,int,float]
 Features      = Union[Feature, Sequence[Feature], Dict[str,Union[int,float]]]
@@ -131,7 +132,7 @@ class VowpalMediator:
                 elif feats.__class__ is int or feats.__class__ is float:
                     yield (ns, [(self._get_namespace_keys(ns,1)[0], feats)])
                 else:
-                    feats = feats.items() if feats.__class__ is dict else enumerate(feats,self._get_namespace_keys(ns,len(feats)))
+                    feats = feats.items() if isinstance(feats, Sparse) else enumerate(feats,self._get_namespace_keys(ns,len(feats)))
                     yield (ns, [f"{k}={v}" if v.__class__ is str else (k, v) for k,v in feats if v!= 0])
         else: #pragma: no cover
             #the strange type checks below were faster than traditional methods when performance testing
@@ -142,7 +143,7 @@ class VowpalMediator:
                     yield (ns, [f"{self._get_namespace_keys(ns,1)[0]}={feats}"])
                 elif feats.__class__ is int or feats.__class__ is float:
                     yield (ns, [(self._get_namespace_keys(ns,1)[0], feats)])
-                elif feats.__class__ is dict:
+                elif isinstance(feats,Sparse):
                     yield (ns, feats)
                 else:
                     d={}
@@ -172,7 +173,7 @@ class VowpalMediator:
 
         return keys
 
-class VowpalArgsLearner(Learner):
+class VowpalLearner(Learner):
     """A friendly wrapper around Vowpal Wabbit's python interface to support CB learning.
 
     Remarks:
@@ -227,7 +228,7 @@ class VowpalArgsLearner(Learner):
         return " ".join(options)
 
     def __init__(self, args: str = "--cb_explore_adf --epsilon 0.05 --interactions ax --interactions axx --ignore_linear x --random_seed 1 --quiet", vw: VowpalMediator = None) -> None:
-        """Instantiate a VowpalArgsLearner.
+        """Instantiate a VowpalLearner.
 
         Args:
             args: Command line arguments to instantiate a Vowpal Wabbit contextual bandit learner. For
@@ -240,7 +241,7 @@ class VowpalArgsLearner(Learner):
         """
 
         if "--cb" not in args:
-            raise CobaException("VowpalArgsLearner was instantiated without a cb flag. One of the cb flags must be defined.")
+            raise CobaException("VowpalLearner was instantiated without a cb flag. One of the cb flags must be defined.")
 
         self._args    = args
         self._explore = "--cb_explore" in args
@@ -258,10 +259,10 @@ class VowpalArgsLearner(Learner):
         self._vw = vw or VowpalMediator()
 
     @property
-    def params(self) -> Dict[str, Any]:
+    def params(self) -> Mapping[str, Any]:
         return {"family": "vw", 'args': self._args.replace("--quiet","").strip()}
 
-    def predict(self, context: Context, actions: Sequence[Action]) -> Tuple[Probs, Info]:
+    def predict(self, context: Context, actions: Sequence[Action]) -> Probs:
 
         if not self._vw.is_initialized and self._adf:
             self._vw.init_learner(self._args, 4)
@@ -285,13 +286,12 @@ class VowpalArgsLearner(Learner):
         if not self._adf and len(actions) != self._n_actions:
             raise CobaException("The number of actions doesn't match the `--cb` action count given in args.")
 
-        info = (actions if self._adf else self._actions)
-
         context = {'x':self._flat(context)}
         adfs    = None if not self._adf else [{'a':self._flat(action)} for action in actions]
 
         if self._adf and self._explore:
             probs = self._vw.predict(self._vw.make_examples(context, adfs, None))
+            if probs is None: probs = [1/len(actions)] * len(actions)
 
         if self._adf and not self._explore:
             losses    = self._vw.predict(self._vw.make_examples(context,adfs, None))
@@ -302,14 +302,15 @@ class VowpalArgsLearner(Learner):
 
         if not self._adf and self._explore:
             probs = self._vw.predict(self._vw.make_example(context, None))
+            if probs is None: probs = [1/len(actions)] * len(actions)
 
         if not self._adf and not self._explore:
             index = self._vw.predict(self._vw.make_example(context, None))
             probs = [ int(i==index) for i in range(1,len(actions)+1) ]
 
-        return probs, info
+        return Probs(probs)
 
-    def learn(self, context: Context, action: Action, reward: float, probability: float, info: Info) -> None:
+    def learn(self, context: Context, actions:Actions, index: int, reward: float, probability: float) -> None:
 
         if not self._vw.is_initialized and self._adf:
             self._vw.init_learner(self._args, 4)
@@ -320,9 +321,8 @@ class VowpalArgsLearner(Learner):
         if not self._vw.is_initialized and not self._adf and not self._n_actions:
             raise CobaException("When using `cb` without `adf` predict must be called before learn to initialize the vw learner")
 
-        actions = info
-        labels  = self._labels(actions, action, reward, probability)
-        label   = labels[actions.index(action)]
+        labels  = self._labels(actions, index, reward, probability)
+        label   = labels[index]
 
         context = {'x':self._flat(context)}
         adfs    = None if not self._adf else [{'a':self._flat(action)} for action in actions]
@@ -332,14 +332,14 @@ class VowpalArgsLearner(Learner):
         else:
             self._vw.learn(self._vw.make_example(context, label))
 
-    def _labels(self,actions,action,reward:float,prob:float) -> Sequence[Optional[str]]:
-        return [ f"{i+1}:{round(-reward,5)}:{round(prob,5)}" if a == action else None for i,a in enumerate(actions)]
+    def _labels(self,actions,index,reward:float,prob:float) -> Sequence[Optional[str]]:
+        return [ f"{i+1}:{round(-reward,5)}:{round(prob,5)}" if i == index else None for i in range(len(actions))]
 
     def _flat(self,features:Any) -> Any:
         return list(Flatten().filter([features]))[0]
 
-class VowpalEpsilonLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalEpsilonLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
         __ https://github.com/VowpalWabbit/vowpal_wabbit/wiki/Contextual-Bandit-algorithms
@@ -364,10 +364,10 @@ class VowpalEpsilonLearner(VowpalArgsLearner):
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
 
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalSoftmaxLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalSoftmaxLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
         __ https://github.com/VowpalWabbit/vowpal_wabbit/wiki/Contextual-Bandit-algorithms
@@ -395,10 +395,10 @@ class VowpalSoftmaxLearner(VowpalArgsLearner):
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalBagLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalBagLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
         __ https://github.com/VowpalWabbit/vowpal_wabbit/wiki/Contextual-Bandit-algorithms
@@ -424,10 +424,10 @@ class VowpalBagLearner(VowpalArgsLearner):
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalCoverLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalCoverLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
     For more information on this algorithm see Agarwal et al. (2014).
@@ -459,10 +459,10 @@ class VowpalCoverLearner(VowpalArgsLearner):
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
 
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalRegcbLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalRegcbLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
     References:
@@ -493,10 +493,10 @@ class VowpalRegcbLearner(VowpalArgsLearner):
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalSquarecbLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalSquarecbLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
     References:
@@ -536,15 +536,15 @@ class VowpalSquarecbLearner(VowpalArgsLearner):
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
-class VowpalOffPolicyLearner(VowpalArgsLearner):
-    """A wrapper around VowpalArgsLearner that provides more documentation. For more
+class VowpalOffPolicyLearner(VowpalLearner):
+    """A wrapper around VowpalLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
 
         This wrapper in particular performs policy learning without any exploration. This is
         only correct when training examples come from a logging policy so that any exploration
-        on our part is ignored.
+        on our part would be irrelevant.
 
         __ https://github.com/VowpalWabbit/vowpal_wabbit/wiki/Contextual-Bandit-algorithms
     """
@@ -565,4 +565,4 @@ class VowpalOffPolicyLearner(VowpalArgsLearner):
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
         vw            = kwargs.pop('vw',None)
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
+        super().__init__(VowpalLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
