@@ -9,11 +9,15 @@ in coba.pipes can help one understand how to best take advantage what Coba has t
 from coba.pipes.primitives import Filter, Source, Sink
 from coba.pipes.multiprocessing import Multiprocessor
 
-from coba.pipes.filters import Take, Shuffle, Drop, Structure, Identity, Flatten, Default, Reservoir
-from coba.pipes.filters import Encode, JsonDecode, JsonEncode
+from coba.pipes.filters import Take, Shuffle, Structure, Identity, Flatten, Default, Reservoir
+from coba.pipes.filters import Encode, JsonDecode, JsonEncode, Cache
 
+from coba.pipes.rows    import LabelRows, EncodeRows, HeadRows, DropRows, EncodeCatRows, Categorical
+from coba.pipes.rows    import LazyDense, EncodeDense, LabelDense, KeepDense, HeadDense
+from coba.pipes.rows    import LazySparse, EncodeSparse, LabelSparse, DropSparse
 from coba.pipes.readers import ManikReader, LibsvmReader, CsvReader, ArffReader
-from coba.pipes.sources import NullSource, DiskSource, ListSource, QueueSource, HttpSource, LambdaSource, UrlSource
+from coba.pipes.sources import NullSource, IdentitySource, DiskSource, IterableSource
+from coba.pipes.sources import QueueSource, HttpSource, LambdaSource, UrlSource
 from coba.pipes.sinks   import NullSink, ConsoleSink, DiskSink, ListSink, QueueSink, LambdaSink
 
 from coba.pipes.core import Pipes, Foreach, QueueIO, SourceFilters
@@ -31,17 +35,17 @@ __all__ = [
     "ArffReader",
     "LibsvmReader",
     "ManikReader",
+    "IdentitySource",
     "UrlSource",
     "NullSource",
     "DiskSource",
-    "ListSource",
+    "IterableSource",
     "QueueSource",
     "HttpSource",
     "LambdaSource",
     "Encode",
     "Flatten",
     "Default",
-    "Drop",
     "Structure",
     "Identity",
     "Take",
@@ -54,5 +58,21 @@ __all__ = [
     "QueueSink",
     "LambdaSink",
     "QueueIO",
-    "SourceFilters"
+    "SourceFilters",
+    "Cache",
+    "LazyDense",
+    "LazySparse",
+    "EncodeDense",
+    "LabelDense",
+    "KeepDense",
+    "HeadDense",
+    "EncodeSparse",
+    "LabelSparse",
+    "DropSparse",
+    "LabelRows", 
+    "EncodeRows", 
+    "HeadRows",
+    "DropRows",
+    "EncodeCatRows",
+    "Categorical",
 ]
\ No newline at end of file
