@@ -2,12 +2,14 @@ import json
 import math
 import copy
 
-from collections import defaultdict
+from collections import defaultdict, abc
 from itertools import islice, chain
-from typing import Iterable, Any, Sequence, Dict, Callable, Optional, Union, MutableSequence, MutableMapping
+from typing import Iterable, Any, Sequence, Mapping, Optional, Union
 
 from coba.random import CobaRandom
 from coba.encodings import Encoder, CobaJsonEncoder, CobaJsonDecoder
+from coba.utilities import peek_first
+from coba.primitives import Sparse,Dense
 
 from coba.pipes.primitives import Filter
 
@@ -17,7 +19,7 @@ class Identity(Filter[Any, Any]):
         return item
 
     @property
-    def params(self) -> Dict[str, Any]:
+    def params(self) -> Mapping[str, Any]:
         return {}
 
 class Shuffle(Filter[Iterable[Any], Sequence[Any]]):
@@ -39,7 +41,7 @@ class Shuffle(Filter[Iterable[Any], Sequence[Any]]):
         return CobaRandom(self._seed).shuffle(list(items))
 
     @property
-    def params(self) -> Dict[str, Any]:
+    def params(self) -> Mapping[str, Any]:
         return { "shuffle": self._seed }
 
 class Take(Filter[Iterable[Any], Sequence[Any]]):
@@ -63,7 +65,7 @@ class Take(Filter[Iterable[Any], Sequence[Any]]):
         return islice(items, self._count)
 
     @property
-    def params(self) -> Dict[str, Any]:
+    def params(self) -> Mapping[str, Any]:
         return { "take": self._count }
 
 class Reservoir(Filter[Iterable[Any], Sequence[Any]]):
@@ -94,7 +96,7 @@ class Reservoir(Filter[Iterable[Any], Sequence[Any]]):
         self._seed  = seed
 
     @property
-    def params(self) -> Dict[str, Any]:
+    def params(self) -> Mapping[str, Any]:
         return { "reservoir_count": self._count, "reservoir_seed": self._seed }
 
     def filter(self, items: Iterable[Any]) -> Sequence[Any]:
@@ -120,12 +122,12 @@ class Reservoir(Filter[Iterable[Any], Sequence[Any]]):
         except StopIteration:
             pass
 
-        return Take(self._count).filter(reservoir)
+        return reservoir
 
 class JsonEncode(Filter[Any, str]):
     """A filter which turn a Python object into JSON strings."""
 
-    def _min(self,obj):
+    def _min(self, obj):
         #WARNING: This method doesn't handle primitive types such int, float, or str. We handle this shortcoming
         #WARNING: by making sure no primitive type is passed to this method in filter. Accepting the shortcoming
         #WARNING: improves the performance of this method by a few percentage points.
@@ -186,35 +188,50 @@ class Flatten(Filter[Iterable[Any], Iterable[Any]]):
 
     def filter(self, data: Iterable[Any]) -> Iterable[Any]:
 
-        for row in data:
+        first, data = peek_first(data)
 
-            if isinstance(row,dict):
-                row = dict(row)
-                for k in list(row.keys()):
-                    if isinstance(row[k],(list,tuple)):
-                        row.update([(f"{k}_{i}", v) for i,v in enumerate(row.pop(k))])
+        if isinstance(first,abc.MutableSequence):
+            first_type = "list" 
+        elif isinstance(first,Dense):
+            first_type = "tuple"
+        elif isinstance(first,Sparse):
+            first_type = "dict"
+        else:
+            first_type = None
 
-            elif isinstance(row,list):
-                row = list(row)
-                for k in reversed(range(len(row))):
-                    if isinstance(row[k],(list,tuple)):
-                        for v in reversed(row.pop(k)):
-                            row.insert(k,v)
+        is_flattable = lambda v: isinstance(v,Dense) or not isinstance(v,str) and (isinstance(v,abc.Iterable))
 
-            elif isinstance(row,tuple):
-                row = list(row)
-                for k in reversed(range(len(row))):
-                    if isinstance(row[k],(list,tuple)):
-                        for v in reversed(row.pop(k)):
-                            row.insert(k,v)
-                row = tuple(row)
+        if first_type in ["list","tuple"]:
+            flattable = [ is_flattable(v) for v in first ]
+            any_flattable = any(flattable)
+        elif first_type == "dict":
+            #we assume that flattable entries will be populated in every row even though the dataset is sparse
+            flattable = {k:[f"{k}_{i}" for i in range(len(v))] for k,v in first.items() if is_flattable(v) }
+            any_flattable = bool(flattable)
+        else:
+            flattable = []
+            any_flattable = False
 
-            yield row
+        def flatter_list(row):
+            for f,r in zip(flattable,row):
+                if f: yield from r 
+                else: yield r
 
-class Encode(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Union[MutableSequence,MutableMapping]]]):
+        if not any_flattable or first_type is None:
+            yield from data
+        elif first_type == "list":
+            yield from (list(flatter_list(row)) for row in data)
+        elif first_type == "tuple":
+            yield from (tuple(flatter_list(row)) for row in data)
+        elif first_type == "dict":
+            for row in data:
+                #in-line dict comprehension was faster than a generator like we did with list
+                yield {k:v for k,v in row.items() for k,v in ( zip(flattable[k],v) if k in flattable else ((k,v),)) if v != 0 }
+
+class Encode(Filter[Iterable[Union[Sequence,Mapping]], Iterable[Union[Sequence,Mapping]]]):
     """A filter which encodes features in table shaped data."""
 
-    def __init__(self, encoders:Dict[Union[str,int],Encoder], fit_using:int = None, missing_val: str = None):
+    def __init__(self, encoders:Mapping[Any,Encoder], fit_using:int = None, missing_val: str = None):
         """Instantiate an Encode filter.
 
         Args:
@@ -226,7 +243,7 @@ class Encode(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Un
         self._fit_using   = fit_using
         self._missing_val = missing_val
 
-    def filter(self, items: Iterable[Union[MutableSequence,MutableMapping]]) -> Iterable[Union[MutableSequence,MutableMapping]]:
+    def filter(self, items: Iterable[Union[Sequence,Mapping]]) -> Iterable[Union[Sequence,Mapping]]:
 
         items = iter(items) # this makes sure items are pulled out for fitting
 
@@ -236,7 +253,7 @@ class Encode(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Un
         except StopIteration:
             return []
 
-        is_dense = not isinstance(first_item,dict)
+        is_dense = isinstance(first_item,Dense)
 
         if not is_dense:
             encoders = self._encoders
@@ -264,38 +281,14 @@ class Encode(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Un
             encoders[k] = encoders[k].fit(v)
 
         for item in chain(items_for_fitting, items):
+            item = list(item) if is_dense else dict(item)
             for k,v in encoders.items():
                 if is_dense or k in item:
                     val = item[k]
                     item[k] = encoders[k].encode(val) if val not in ['',self._missing_val] else val
             yield item
 
-class Drop(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Union[MutableSequence,MutableMapping]]]):
-    """A filter which drops rows and columns from in table shaped data."""
-
-    def __init__(self,
-        drop_cols: Sequence[Union[str,int]] = [],
-        drop_row: Callable[[Union[MutableSequence,MutableMapping]], bool] = None) -> None:
-        """Instantiate a Drop filter.
-
-        Args:
-            drop_cols: Feature names which should be dropped.
-            drop_row: A function that accepts a row and returns True if it should be dropped.
-        """
-
-        self._drop_cols = sorted(drop_cols, reverse=True)
-        self._drop_row  = drop_row or (lambda r: False)
-
-    def filter(self, data: Iterable[Union[MutableSequence,MutableMapping]]) -> Iterable[Union[MutableSequence,MutableMapping]]:
-
-        keep_row = lambda r: not self._drop_row(r) if self._drop_row else True
-        for row in filter(keep_row, data):
-            if row is not None:
-                for col in self._drop_cols:
-                    del row[col]
-            yield row
-
-class Structure(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Any]]):
+class Structure(Filter[Iterable[Union[Sequence,Mapping]], Iterable[Any]]):
     """A filter which restructures rows in table shaped data."""
 
     def __init__(self, structure: Sequence[Any]) -> None:
@@ -305,6 +298,7 @@ class Structure(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable
             structure: The structure that each row should be reformed into. Items in the structure should be
                 feature names. A None value indicates the location that all left-over features will be placed.
         """
+
         self._structure = []
         stack = [structure]
         while stack:
@@ -320,8 +314,14 @@ class Structure(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable
             else:
                 self._structure.insert(0,item)
 
-    def filter(self, data: Iterable[Union[MutableSequence,MutableMapping]]) -> Iterable[Any]:
+    def filter(self, data: Iterable[Union[Sequence,Mapping]]) -> Iterable[Any]:
+        first, data = peek_first(data)
+        is_dense = isinstance(first, Dense)
+
         for row in data:
+
+            row = list(row) if is_dense else dict(row.items())
+
             stack   = []
             working = []
             for item in self._structure:
@@ -343,10 +343,10 @@ class Structure(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable
 
             yield working[0]
 
-class Default(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[Union[MutableSequence,MutableMapping]]]):
+class Default(Filter[Iterable[Union[Sequence,Mapping]], Iterable[Union[Sequence,Mapping]]]):
     """A filter which sets default values for row features in table shaped data."""
 
-    def __init__(self, defaults: Dict[Any, Any]) -> None:
+    def __init__(self, defaults: Mapping[Any, Any]) -> None:
         """Instantiate a Default filter.
 
         Args:
@@ -354,12 +354,35 @@ class Default(Filter[Iterable[Union[MutableSequence,MutableMapping]], Iterable[U
         """
         self._defaults = defaults
 
-    def filter(self, data: Iterable[Union[MutableSequence,MutableMapping]]) -> Iterable[Union[MutableSequence,MutableMapping]]:
+    def filter(self, data: Iterable[Union[Sequence,Mapping]]) -> Iterable[Union[Sequence,Mapping]]:
 
         for row in data:
 
-            if isinstance(row,dict):
+            if isinstance(row, Sparse):
+                row = dict(row)
                 for k,v in self._defaults.items():
                     if k not in row: row[k] = v
 
             yield row
+
+class Cache(Filter[Iterable[Any], Iterable[Any]]):
+
+    def __init__(self,n_slice:int=25) -> None:
+        self._cache = None
+        self._n_slice = n_slice
+
+    def filter(self, items: Iterable[Any]) -> Iterable[Any]:
+        n_slice = self._n_slice
+
+        if self._cache:
+            yield from self._cache
+
+        else:
+            temp_cache = []
+            items = iter(items)
+            current = list(islice(items,n_slice))
+            while current:
+                temp_cache.extend(current)
+                yield from current
+                current = list(islice(items,n_slice))
+            self._cache = temp_cache
