@@ -2,7 +2,7 @@ import requests
 import gzip
 
 from queue import Queue
-from typing import Callable, Iterable, Sequence, Any, Union
+from typing import Callable, Iterable, Any, Union, Mapping
 from coba.backports import Literal
 
 from coba.exceptions import CobaException
@@ -14,6 +14,28 @@ class NullSource(Source[Any]):
     def read(self) -> Iterable[Any]:
         return []
 
+class IdentitySource(Source[Any]):
+    """A source which reads from an iterable."""
+
+    def __init__(self, item: Any, params: Mapping[str,Any] = None):
+        """Instantiate an IterableSource.
+
+        Args:
+            item: The item to return from read.
+            params: Teh params descirbing the source.
+        """
+        self._item = item
+        self._params = params or {}
+
+    @property
+    def params(self) -> Mapping[str, Any]:
+        return self._params
+
+    def read(self) -> Iterable[Any]:
+        return self._item
+
+
+
 class DiskSource(Source[Iterable[str]]):
     """A source which reads a file from disk.
 
@@ -100,21 +122,19 @@ class HttpSource(Source[Union[requests.Response, Iterable[str]]]):
         response = requests.get(self._url, stream=True) #by default this includes the header accept-encoding gzip and deflate
         return response if self._mode == "response" else response.iter_lines(decode_unicode=True)
 
-class ListSource(Source[Iterable[Any]]):
-    """A source which reads from a list."""
+class IterableSource(Source[Iterable[Any]]):
+    """A source which reads from an iterable."""
 
-    def __init__(self, items: Sequence[Any]=None):
-        """Instantiate a ListSource.
+    def __init__(self, iterable: Iterable[Any]=None):
+        """Instantiate an IterableSource.
 
         Args:
-            items: The list object we should read from.
+            iterable: The iterable we should read from.
         """
-
-        self.items = [] if items is None else items
+        self.iterable = [] if iterable is None else iterable
 
     def read(self) -> Iterable[Any]:
-        for item in self.items:
-            yield item
+        return self.iterable
 
 class LambdaSource(Source[Any]):
     """A source which reads from a callable method."""
