@@ -15,7 +15,7 @@ import math
 import itertools
 import time
 
-from typing import Optional, Sequence, Any, List
+from typing import Optional, Iterable, Sequence, Any
 
 class CobaRandom:
     """A random number generator that is consistent across python implementations."""
@@ -37,22 +37,17 @@ class CobaRandom:
             and good lattice structure." Mathematics of Computation 68.225 (1999): 249-260.
         """
 
-        self._m = 2**30
-        self._a = 116646453
-        self._c = 9
-
-        self._times = [0,0,0]
-
-        self._m_is_power_of_2 = (self._m % 2) == 0
-        self._m_minus_1       = self._m-1
-
         if isinstance(seed,int) or (isinstance(seed,float) and seed.is_integer()):
-            self._seed = int(seed)
+            seed = int(seed)
         else:
-            self._seed = int.from_bytes(str(seed or time.time()).encode('utf-8'),"big") % 2**20
+            seed = int.from_bytes(str(seed or time.time()).encode('utf-8'),"big") % 2**20
+
+        self._seed = seed
+        self._randu = self._next_uniform(116646453,seed,9,2**30)
+        self._randg = self._next_gaussian()
 
     def randoms(self, n:int, min:float=0, max:float=1) -> Sequence[float]:
-        """Generate `n` uniform random numbers in [`min`,`max`].
+        """Generate `n` uniform random numbers in [`min`,`max`).
 
         Args:
             n: How many random numbers should be generated.
@@ -62,8 +57,18 @@ class CobaRandom:
         Returns:
             The `n` generated random numbers in [`min`,`max`].
         """
+        if (n is not None) and (n < 0 or not isinstance(n, int)):
+            raise ValueError("n must be an integer greater than or equal to 0")
 
-        return [min+(max-min)*number/self._m_minus_1 for number in self._next(n)]
+        if min == 0 and max == 1:
+            iterable = itertools.islice(self._randu,n)
+        elif min == 0:
+            iterable = ( max*r for r in itertools.islice(self._randu,n) )
+        else:
+            r_range = max-min
+            iterable = (min+r_range*r for r in itertools.islice(self._randu,n))
+        
+        return list(iterable) if n is not None else iterable
 
     def random(self, min:float=0, max:float=1) -> float:
         """Generate a uniform random number in [`min`,`max`].
@@ -75,7 +80,7 @@ class CobaRandom:
         Returns:
             The generated random number in [`min`,`max`].
         """
-        return self.randoms(1,min,max)[0]
+        return min+(max-min)*next(self._randu)        
 
     def shuffle(self, sequence: Sequence[Any]) -> Sequence[Any]:
         """Shuffle the order of items in a sequence.
@@ -92,16 +97,14 @@ class CobaRandom:
         """
 
         n = len(sequence)
-        r = self.randoms(n)
+        if n < 2: return sequence
         l = list(sequence)
 
-        for i in range(0,n-1):
-
-            j = int(i + (r[i] * (n-i))) # i <= j <= n
-            j = min(j, n-1)             # i <= j <= n-1 (min handles the edge case of r[i]==1 which would make j=n)
-
+        #i goes from 0 to n-2
+        #j is always i <= j < n
+        for i,r in itertools.islice(enumerate(self._randu),n-1):
+            j = i+int(r*(n-i)) 
             l[i], l[j] = l[j], l[i]
-
         return l
 
     def randint(self, a:int, b:int) -> int:
@@ -112,9 +115,9 @@ class CobaRandom:
             b: The inclusive upper bound for the random integer.
         """
 
-        return min(int((b-a+1) * self.random()), b-a) + a
+        return a+int((b-a+1)*next(self._randu))
 
-    def randints(self, n:int, a:int, b:int) -> int:
+    def randints(self, n:int, a:int, b:int) -> Sequence[int]:
         """Generate `n` uniform random integers in [a, b].
 
         Args:
@@ -122,9 +125,12 @@ class CobaRandom:
             a: The inclusive lower bound for the random integer.
             b: The inclusive upper bound for the random integer.
         """
-
-        return [min(int((b-a+1) * r), b-a) + a for r in self.randoms(n)]
-
+        b=b+1
+        if a == 0:
+            return [int(b*r) for r in itertools.islice(self._randu,n)]
+        else:
+            r_range = b-a
+            return [int(r_range*r) + a for r in itertools.islice(self._randu,n)]
 
     def choice(self, seq: Sequence[Any], weights:Sequence[float] = None) -> Any:
         """Choose a random item from the given sequence.
@@ -135,16 +141,15 @@ class CobaRandom:
         """
 
         if weights is None:
-            return seq[self.randint(0, len(seq)-1)]
+            return seq[int(len(seq)*next(self._randu))]
         else:
 
-            if sum(weights) == 0:
-                raise ValueError("The sum of weights cannot be zero.")
-
             cdf = list(itertools.accumulate(weights))
-            rng = self.random() * sum(weights)
+            if cdf[-1] == 0: raise ValueError("The sum of weights cannot be zero.")
+            rng = next(self._randu) * cdf[-1]
 
-            return seq[[ rng <= c for c in cdf].index(True)]
+            for s,c in zip(seq,cdf):
+                if rng <= c: return s
 
     def gauss(self, mu:float=0, sigma:float=1) -> float:
         """Generate a random number from N(mu,sigma).
@@ -169,52 +174,40 @@ class CobaRandom:
         Returns:
             The `n` random numbers drawn from N(mu,sigma).
         """
-        #Box-muller transform. Generates independent pseudo-normal random variable pairs.
+        return [mu+sigma*g for g in itertools.islice(self._randg,n) ]
 
-        if n == 0: return []
+    def _next_uniform(self, a, s, c, m) -> Iterable[float]:
+        """Generate uniform random numbers in [0,1).
 
-        n_pairs_to_generate = math.ceil(n/2)
+        Random numbers are generated using a linear congruential generator.
+        """
+        m_1 = m-1
+        while True:
+            #when m is a power of 2 
+            #this is equal to modulo m
+            s = (a * s + c) & (m_1)
+            yield s/m
 
-        Rs = [math.sqrt(-2*math.log(u)) for u in self.randoms(n_pairs_to_generate)]
-        Ss = [2*math.pi*u               for u in self.randoms(n_pairs_to_generate)]
-
-        Ns = []
-        for R,S in zip(Rs,Ss):
-            Ns.append(mu+sigma*R*math.cos(S))
-            Ns.append(mu+sigma*R*math.sin(S))
-
-        if len(Ns) > n: Ns.pop()
-
-        return Ns
-
-    def _next(self, n: int) -> Sequence[int]:
-        """Generate `n` uniform random numbers in [0,m-1]
-
-        Random numbers are generated using a linear congruential generator
-
-        Args:
-            n: The number of random numbers to generate.
-
-        Returns:
-            The `n` generated random numbers in [0,m-1].
+    def _next_gaussian(self) -> Iterable[float]:
+        """Generate `n` gaussian random numbers in N(0,1).
+        
+        Random numbers are generated using the Box-Muller transform.
         """
 
-        if n < 0 or not isinstance(n, int):
-            raise ValueError("n must be an integer greater than or equal 0")
+        sqrt = math.sqrt
+        log  = math.log
+        pi   = math.pi
+        cos  = math.cos
+        sin  = math.sin
 
-        numbers: List[int] = []
+        while True:
+            R = sqrt(-2*log(next(self._randu)))
+            S = 2*pi*next(self._randu)
+            yield R*cos(S)
+            yield R*sin(S)
 
-        for _ in range(n):
-
-            #when _m is a power of 2 these two statements are equal to eachother
-            if self._m_is_power_of_2:
-                self._seed = int((self._a * self._seed + self._c) & (self._m_minus_1))
-            else:
-                self._seed = int((self._a * self._seed + self._c) % self._m)
-
-            numbers.append(self._seed)
-
-        return numbers
+    def __reduce__(self):
+        return (CobaRandom,(self._seed,))
 
 _random = CobaRandom()
 
@@ -291,6 +284,17 @@ def randint(a:int, b:int) -> int:
 
     return _random.randint(a,b)
 
+def randints(n:int, a:int, b:int) -> Sequence[int]:
+    """Generate `n` uniform random integers in [a, b].
+
+    Args:
+        n: The number of random integers to generate.
+        a: The inclusive lower bound for the random integer.
+        b: The inclusive upper bound for the random integer.
+    """
+
+    return _random.randints(n,a,b)
+
 def choice(seq: Sequence[Any], weights:Sequence[float]=None) -> Any:
     """Choose a random item from the given sequence.
 
